@@ -1,3 +1,5 @@
+//! Widget property types
+
 use crate::{Prefab, PrefabError, PrefabValue};
 use serde::{Deserialize, Serialize};
 use std::{
@@ -95,7 +97,7 @@ where
         Box::new(self.clone())
     }
 
-    fn as_any(&self) -> &dyn std::any::Any {
+    fn as_any(&self) -> &dyn Any {
         self
     }
 }
@@ -117,12 +119,12 @@ where
         Box::new(self.clone())
     }
 
-    fn as_any(&self) -> &dyn std::any::Any {
+    fn as_any(&self) -> &dyn Any {
         self
     }
 }
 
-pub trait PropsData: std::fmt::Debug + Send + Sync {
+pub trait PropsData: Any + std::fmt::Debug + Send + Sync {
     fn clone_props(&self) -> Box<dyn PropsData>;
     fn as_any(&self) -> &dyn Any;
 }
@@ -165,6 +167,17 @@ impl Props {
         }
     }
 
+    pub fn consume_unwrap_cloned<T>(&mut self) -> Result<T, PropsError>
+    where
+        T: 'static + PropsData + Clone,
+    {
+        if let Some(data) = self.consume::<T>()?.as_any().downcast_ref::<T>() {
+            Ok(data.clone())
+        } else {
+            Err(PropsError::CouldNotReadData)
+        }
+    }
+
     pub fn read<T>(&self) -> Result<&T, PropsError>
     where
         T: 'static + PropsData,
@@ -326,6 +339,11 @@ impl From<&Self> for Props {
     }
 }
 
+/// Macro for implementing [`PropsData`].
+///
+/// You may prefer to use the [derive macro][`macro@crate::PropsData`] instead, but in case of
+/// auto-implementing PropsData and Prefab traits for remote or std types, you might find this macro
+/// useful.
 #[macro_export]
 macro_rules! implement_props_data {
     ($type_name:ty) => {
@@ -359,5 +377,7 @@ implement_props_data!(u64);
 implement_props_data!(u128);
 implement_props_data!(f32);
 implement_props_data!(f64);
+implement_props_data!(isize);
+implement_props_data!(usize);
 implement_props_data!(bool);
 implement_props_data!(String);
